<?php
/*
Copyright (C) 2006 Daniel Webb

This file is part of bot-trap.

bot-trap is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

bot-trap is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with bot-trap; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

Version 12.0.0.7
Modified by Steven Davis for TNG
*/

function is_blacklisted ($ip)
{
  $badbot = 0;
  require '../bot-trap/settings.php';

  /* Create .htaccess file in TNG root if it's not there already */
  if (!is_file($htaccess_file)) 
    { 
      touch($htaccess_file);
      if (!is_file($htaccess_file)) 
        { 
          echo "<p>Can't create $htaccess_file, do you have permissions?</p>";
          return;
        }
      else {
        $fp = fopen($htaccess_file, 'w');
        fwrite($fp, "# Access file for Apache 2.4" . "\n" . "<RequireAll>" . "\n" . "	Require all granted" . "\n" . "    # Anthing below this line was added by Bot-trap" . "\n" . "</RequireAll>");
        fclose($fp);
        chmod($htaccess_file, 0664);
      }
    }

  /* Create blacklist file if its not there already */
  if (!is_file($blacklist_file)) 
    { 
      touch($blacklist_file); 
      if (!is_file($blacklist_file)) 
        {
          echo "<p>Can't create $blacklist_file, do you have permissions?</p>";
          return;
        }
    }

  /* scan the blacklist file to see if this IP address is there */
  $fp = fopen($blacklist_file, "r") or die ("Error opening file $blacklist_file <br>\n");
  if (flock($fp, LOCK_SH)) 
    {
      while ($line = fgets($fp,255))  
        {
          if (preg_match('/^'.$ip.'/',$line)) { $badbot++; }
        }
      flock($fp, LOCK_UN); 
    }
  else 
    {
      echo "<p>Can't get lock for $blacklist_file</p>";
    }
  fclose($fp);
  return $badbot;
}
?> 

<?php
    $referer = $_SERVER['HTTP_REFERER'];
		$referer = str_replace("&tree=$tree","",$referer);
		$referer = str_replace($tngdomain,".",$referer);
		$referer = str_replace("&&","&",$referer);
		$referer = str_replace("&amp;$","",$referer);
		$referer = preg_replace("/password=.*/","",$referer);

		// It appends the referrer page info to the log line

		if ($referer != '') {
			$string .=trim(strip_tags(htmlspecialchars($referer))); 
		}
		if( strlen($string) > 400 ) $string = substr($string,0,400);

//  User agent string detection
  $http_referrer = $_SERVER['HTTP_USER_AGENT'];

function add_blacklist ($ip, $req_method, $req_uri, $server_protocol,
    $http_referrer, $http_user_agent)
{
  require '../bot-trap/settings.php';

  /* Check to make sure they aren't blacklisted already */
  /* Note: this function also makes sure $blacklist_file and $htaccess_file */
  /*       exist via is_blacklisted() */
  if (is_blacklisted($ip)) { return; }

  /* append bad bot address data to blacklist log file: */
  $timestamp = time();
  $datum = date("Y-m-d (D) H:i:s",$timestamp);
  if (!is_file("$blacklist_file"))
    {
      echo "<p>Can't open $blacklist_file, do you have permissions to it?</p>";
      return;
    }
  $fp = fopen($blacklist_file,'a+');
  if (flock($fp, LOCK_EX))
    {
      fwrite($fp,"$ip - - [$datum] \"$req_method $req_uri $server_protocol\" $http_referrer $http_user_agent\n");
      flock($fp, LOCK_UN);
    }
  else
    {
      echo "<p>Couldn't get lock for $blacklist_file</p>";
    }
  fclose($fp);
  
  /* append to .htaccess file for automatic denial: */
  /*  Code changed for Apache 2.4 with Authorization Containers- Read contents of htaccess file */
  $fp = fopen($htaccess_file,'r');
  if (flock($fp, LOCK_EX))
    $lines = file($htaccess_file);
  else {
    return;
  }
  flock($fp, LOCK_UN);
  fclose($fp);

  $key = "</RequireAll>";
  $newline = ("    Require not ip $ip\n");

  if ($backup_htaccess) {
  copy($htaccess_file, $backup_file); /* Back up original htaccess file as a copy */
  }
  /* Write lines back to htaccess */
  $fp = fopen($htaccess_file,'w');
  if (flock($fp, LOCK_EX))
    {
      foreach($lines as $line){
        if (stristr($line,$key))  //look for $key in each line
          {
            fwrite($fp, $newline); //insert denied IP address before line with key
          } 
        fwrite($fp, $line); //place $line back in file
        }
    }
  else
    {
      echo "<p>Couldn't get lock for $htaccess_file</p>";
    }
  flock($fp, LOCK_UN);
  fclose($fp);
  /* If backup file setting is false and the file exists, delete the backup file */
  if (!$backup_htaccess && file_exists($backup_file)){
  unlink($backup_file);
  }

  if ($mail_on_add === true)
    {
      /* tell the webmaster about this */
      $hostname = gethostbyaddr($ip); 
      $subject = "Bot-Trap alert: bad robot from $ip ($hostname)";
      $msg = "A bad robot hit $req_uri $datum \n";
      $msg .= "address is $ip, hostname is $hostname, agent is $http_user_agent\n";
      mail($alert_email_to, $subject, $msg, "From: $alert_email_from");
    }
}
?> 

<?php
function remove_line_from_file ($file, $regex)
{
  if (!is_file("$file"))
    {
      echo "<p>Can't open $file, do you have read permissions to it?</p>";
      return;
    }
  $fp = fopen($file,'r');
  $lines = file($file);
  fclose($fp);
    
  $fp = fopen($file, 'w+');
  if (flock($fp, LOCK_EX))
  {
    foreach($lines as $line)
    {
    if (!preg_match("%{$regex}%i",$line)) { fwrite($fp, $line); }
    }
    flock($fp, LOCK_UN);
    fclose($fp);
  }
  elseif (!flock($fp, LOCK_EX))
  {
    echo "<p>Couldn't get lock for $file</p>";
    fclose($fp);
    return;
  }
}
?>

<?php
function remove_blacklist ($ip, $http_user_agent)
{
  require '../bot-trap/settings.php';

  if (is_blacklisted($ip))
    {
      remove_line_from_file($blacklist_file, $ip);
      remove_line_from_file($htaccess_file, $ip);

      if ($mail_on_remove === true)
        {
          /* tell the webmaster about this */
          $timestamp = time();
      	  $hostname = gethostbyaddr($ip); 
          $datum = date("Y-m-d (D) H:i:s",$timestamp);
          $subject = "Bot-Trap alert: $ip ($hostname) removed from ban list";
          $msg .= "address is $ip, hostname is $hostname, agent is $http_user_agent\n";
          mail($alert_email_to, $subject, $msg, "From: $alert_email_from");
        }
    }
}
?>